function create_artefact_subjects()
% Creates subjects with typical artefacts revealed in preprocessing with
% batch_report_snr monitoring
%
%   create_artefact_subjects()
%
% IN
%
% OUT
%   Folders subjectA1, subjectA2, subjectA3 holding different artifacts in
%   structural and functional data are created
%
%   => run batch for each subject in
%   batches/demo02_compare_batch_quality/batch_preproc_fmri_report_quality.m
%
%
% EXAMPLE (spoiler alert!):
%   These are the modifications done to each subject:
%   subjectA1   fmri.nii: volume 2 was rotated by 30 deg (around x-axis),
%               => realignment fails (too big displacement)
%   subjectA2   struct.nii: was rotated by 180 deg (around z-axis), i.e.
%               nose pointing right, not left
%               => segmentation fails (no close match to tissue probability
%                  maps)
%   subjectA3   fmri.nii: shifted by 5 cm in head direction compared to struct
%               => co-registration fails
%               note: this is hard to spot, if you never check_reg fmri.nii
%               and struct.nii _together_!
%
%   See also init_reset_example batch_report_snr
%
% Author: Lars Kasper
% Created: 2015-05-28
% Copyright (C) 2015 TNU, Institute for Biomedical Engineering, University of Zurich and ETH Zurich.
%
% This file is part of the TAPAS PhysIO Toolbox, which is released under the terms of the GNU General Public
% License (GPL), version 3. You can redistribute it and/or modify it under the terms of the GPL
% (either version 3 or, at your option, any later version). For further details, see the file
% COPYING or <http://www.gnu.org/licenses/>.
%
% $Id: init_reset_example.m 734 2015-03-26 15:40:33Z kasperla $


pathCode    = fileparts(mfilename('fullpath'));
pathStudy   = fileparts(pathCode);

for s = 1:3
    dirSubjectArray{s} = ['subjectA' num2str(s)];
end

init_reset_example(2, dirSubjectArray);

fileFmriLast = fullfile(pathStudy, dirSubjectArray{end}, 'fmri', 'fmri.nii');
disp('Initialsing subjectA1...A3 folders');
while ~exist(fileFmriLast, 'file');
    pause(10);
    fprintf('.');
end
disp('');

%% Subject A1: realignment failure by rotating volume 2 by 30 degrees in X

pathSubject = fullfile(pathStudy, dirSubjectArray{1});

% rotate around x-axis (pitch)
% sin(30 deg) = 1/2; cos(30 deg) = sqrt(3)/2
rot30x = [
    1   0           0           0
    0   sqrt(3)/2   -1/2        0
    0   1/2         sqrt(3)/2   0
    0   0           0           1
    ];


% load nifti header, save 4x4 affine matrix it to mat-variable
fileFmri    = fullfile(pathSubject, 'fmri', 'fmri.nii');

V           = spm_vol(fileFmri);


nVolumes    = numel(V);
mat         = repmat(eye(4,4), 1, 1, nVolumes);

for iVol = 1:nVolumes
    mat(:,:,iVol) = V(iVol).mat;
end


% rotate 2nd volume
mat(:,:,2) = rot30x*mat(:,:,2);

save(fullfile(pathSubject, 'fmri', 'fmri.mat'), 'mat');

spm_reslice(fileFmri);

movefile(fullfile(pathSubject, 'fmri', 'rfmri.nii'), ...
    fullfile(pathSubject, 'fmri', 'fmri.nii'))
% delete(fullfile(pathSubject, 'fmri', 'fmri.mat'));
delete(fullfile(pathSubject, 'fmri', 'meanfmri.nii'));

%% Subject A2: Segment fails due to 180 deg rotation around z of structural
% image


pathSubject = fullfile(pathStudy, dirSubjectArray{2});
fileStruct   = fullfile(pathSubject, 'struct', 'struct.nii');

% load file header, rotate affine matrix, write back .nii-file (since
% struct.mat would be ignored for single-volume file

% cos(180 deg) = -1; sin(180 deg) = 0
% rotation around z-axis

rot180z = [
    -1  0   0   0
    0   -1  0   0
    0   0   1   0
    0   0   0   1
    ];

V           = spm_vol(fileStruct);
Y           = spm_read_vols(V);

V.mat       = rot180z*V.mat;

spm_write_vol(V, Y);


%% Subject A3: Co-registration fails by shifting functional images 5 cm
%   head-direction

pathSubject = fullfile(pathStudy, dirSubjectArray{3});

% rotate 2nd volume back
mat(:,:,2) = rot30x'*mat(:,:,2);

% shift all volumes by 50 mm in head direction
shift100xyz          = eye(4);
shift100xyz(1:3,4)   = 100; % in mm

for iVol = 1:nVolumes
    mat(:,:,iVol)  = shift100xyz*mat(:,:,iVol);
end

save(fullfile(pathSubject, 'fmri', 'fmri.mat'), 'mat');

