function [fileMultipleConditions, onsets, durations, names] = get_multiple_conditions_ioio(...
    fileLog, folderMultipleConditions)
% Constructs multiple conditions file for IOIO-experiment session
%
% [onsets, durations, names] = get_multiple_conditions_ioio(...
%     fileLog, fileMultipleConditions)
%
% IN
%   fileLog     log-file with rows of onset times (in seconds) of different
%               conditions
%               e.g. onsets.txt
%   folderMultipleConditions
%               name of output folder for multiple-conditions mat-file
%               default: same as fileLog
%
% OUT
%   fileMultipleConditions 
%               fullfile(folderMultipleConditions',
%               'multiple_conditions.mat'); (needed for SPM-batch editor)
%   onsets      cell(1, nConditions) of stimulus onset vectors
%   durations   cell(1, nConditions) of stimulus duration vectors
%   names       cell(1, nConditions) of condition name strings
%
% EXAMPLE
%   get_multiple_conditions_ioio
%
%   See also
%
% Author: Lars Kasper
% Created: 2015-01-30
% Copyright (C) 2015 TNU, Institute for Biomedical Engineering, University of Zurich and ETH Zurich.
%
% This file is part of the TAPAS PhysIO Toolbox, which is released under the terms of the GNU General Public
% Licence (GPL), version 3. You can redistribute it and/or modify it under the terms of the GPL
% (either version 3 or, at your option, any later version). For further details, see the file
% COPYING or <http://www.gnu.org/licenses/>.
%
% $Id: get_multiple_conditions_ioio.m 734 2015-03-26 15:40:33Z kasperla $


if nargin < 1
    fileLog = '../logs/onsets.txt';
end

% for batch editor inputs (come as cell)
if iscell(fileLog)
    fileLog = fileLog{1};
end

if nargin < 2
    folderMultipleConditions = fileparts(fileLog);
end

if iscell(folderMultipleConditions)
    folderMultipleConditions = folderMultipleConditions{1};
end

% prepend output file with input path, if no other path specified
fileMultipleConditions = fullfile(folderMultipleConditions, ...
    'multiple_conditions.mat');

%% Read all condition names

% read onset names from first, commented line
fid             = fopen(fileLog);
names           = fgets(fid);

% read all words separated by spaces, these are column-names
names           = regexp(names, '\w*', 'match');
nConditions     = numel(names);



%% Read onsets

% create column-read string for all columns to read, 
% e.g %f %f %f %f for 4 columns
str = '';
for c = 1:nConditions
    str = sprintf('%s %s', str, '%f');
end

onsets = textscan(fid, str, 'CommentStyle', '#');
fclose(fid);



%% Create durations & save multiple-conditions file

durations       = cell(1, nConditions);
durations(1:4)  = {0};


save(fileMultipleConditions, 'onsets', 'durations', 'names');
