function init_reset_example(resetMode, dirSubject)
% Sets paths, resets example data, restores original folder structure,
% optionally deletes processed all data
%
%  init_reset_example(resetMode)
%
% Also: SPM batch-files (only .m) are updated with the correct example path
%       given where this code-folder is stored (assuming data in ../raw)
%
% IN
%   resetMode   Mode of initialisation and reset
%           1   init and set paths, copy data,
%               but do not delete/overwrite existing data
%           2   as 1, but delete processed data and re-copy data from raw
%           3   delete all created folders related to 
%               dirSubject (default: subject01)
%           4   restoring initial state of pathStudy:
%               - delete all created subject folders and files, 
%               - delete all ps-files in study folder
%               - keep only code and raw folder
%   
%
% OUT
%
% EXAMPLE
%   init_reset_example
%
%   See also update_absolute_paths
%
% Author: Lars Kasper
% Created: 2015-01-29
% Copyright (C) 2015 TNU, Institute for Biomedical Engineering, University of Zurich and ETH Zurich.
%
% This file is part of the TAPAS PhysIO Toolbox, which is released under the terms of the GNU General Public
% License (GPL), version 3. You can redistribute it and/or modify it under the terms of the GPL
% (either version 3 or, at your option, any later version). For further details, see the file
% COPYING or <http://www.gnu.org/licenses/>.
%
% $Id: init_reset_example.m 734 2015-03-26 15:40:33Z kasperla $
if nargin < 1
    resetMode = 1;
end

doDeleteExistingData    = ismember(resetMode, [2 3]);
doCopyData              = ismember(resetMode, [1 2]);
doOverwriteExistingData = ismember(resetMode, 2);



%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%% 1. Set paths
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

pathSpm = fileparts(which('spm'));

if isempty(pathSpm)
    error('Please add SPM12 to your path');
end

pathCode    = fileparts(mfilename('fullpath'));
pathPhysio  = fileparts(which('tapas_physio_new'));

if isempty(pathPhysio)
    error('Please add PhysIO Toolbox code-folder to your path');
end


addpath(genpath(pathCode));

% this is NOT obsolete, because tapas_physio_job2physio was not found on Matalb 2011a
% even though tapas_physio_new was already in the path
rmpath(pathPhysio);
addpath(pathPhysio);

pathStudy = fileparts(pathCode);

% default subject 
if nargin < 2
    dirSubject = 'subject01';
end

% complete reset of whole folder
if resetMode == 4 
    
    % determine sub-folders of study that are not code/raw, i.e. presumably subject
    % folders
    allDirs = dir(pathStudy);
    allDirs = allDirs(cell2mat({allDirs.isdir}));
    dirSubjectArrayAll = setdiff({allDirs.name}, ...
        {'.', '..', 'code', 'raw'});
    
    init_reset_example(3, dirSubjectArrayAll);
    delete(fullfile(pathStudy, '*.ps'));
    return 
end


% call this function for each subject, if multiple directories given
if iscell(dirSubject)
    for s = 1:numel(dirSubject)
        init_reset_example(resetMode, dirSubject{s});
    end
    return
end

pathSubject = fullfile(pathStudy, dirSubject);
pathRaw     = fullfile(pathStudy, 'raw');
pathFmri    = fullfile(pathSubject, 'fmri');
pathStruct  = fullfile(pathSubject, 'struct');
pathGlm     = fullfile(pathSubject, 'glm');
pathLogs    = fullfile(pathSubject, 'logs');
pathReportQuality = fullfile(pathSubject, 'report_quality');

paths = {
    pathFmri
    pathStruct
    pathGlm
    pathLogs
    pathReportQuality
    };

files = {
    {'fmri.nii'}
    {'struct.nii'}
    {}
    {'onsets.txt', 'phys.log'}
    {}
    };

nPaths = numel(paths);




%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%% 2. Delete and recreate paths, copy files from raw to corresponding folders
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% only in specific sub-folders of subject folder...
for p = 1:nPaths
    
    if doDeleteExistingData
        
        if exist(paths{p}, 'dir')
            rmdir(paths{p}, 's');
        end
    end
    
    if doCopyData
        
        mkdir(paths{p});
        
        % copy all files that have to be copied
        nFiles = numel(files{p});
        for f = 1:nFiles
            fileRaw = fullfile(pathRaw, files{p}{f});
            fileNew = fullfile(paths{p}, files{p}{f});
            if doOverwriteExistingData || ~exist(fileNew, 'file')
                
                if exist(fileRaw, 'file')
                    % simply copy data
                    copyfile(fileRaw, fileNew);
                else
                    % unzip files and move, or copy, if not zipped
                    fileRawZip = [fileRaw, '.gz'];
                    if exist(fileRawZip, 'file')
                        gunzip(fileRawZip);
                        movefile(fileRaw, fileNew)
                    else
                        error(sprintf('File %s not found\n', fileRaw));
                    end
                end
                
            end
        end
    end
end

% delete subject folder...if only raw stuff is to be kept
if resetMode == 3
    if exist(pathSubject, 'dir')
        rmdir(pathSubject, 's');
    end
end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%% 3. Replace selected batches
% NOTE: batch-folder is not replaced, in case people have changed sth there
% manually :-)
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% batch files path on this system
rootBatch       = fullfile(pathSubject, 'batches');
rootBatchRaw    = fullfile(pathRaw, 'batches');

if doCopyData
    
    pathRawBatchArray = dir(rootBatchRaw);
    dirRawBatchArray = ...
        {pathRawBatchArray([pathRawBatchArray.isdir]).name};
    
    nBatchDirs = numel(dirRawBatchArray);
    
    for iBatchDir = 1:nBatchDirs
        
        % source folder of unaltered batch-files, as created on Lars' computer
        dirBatch        = dirRawBatchArray{iBatchDir};        
        pathRawBatch    = fullfile(rootBatchRaw, dirBatch);
        pathBatch       = fullfile(rootBatch, dirBatch);
        
        
        % path to be replaced in batch-files
        pathSubjectReplace = ...
            '/Users/kasperla/Dropbox/Conferences/SPMZurich15/PracticalsPhysIOPreprocessing/example_physio_short/subject01';
        pathSpmReplace = ...
            '/Users/kasperla/Documents/code/matlab/spm12';
        pathStudyReplace = ...
            '/Users/kasperla/Dropbox/Conferences/SPMZurich15/PracticalsPhysIOPreprocessing/example_physio_short';
        dirSubjectReplace = 'subject01';
        
        
        
        % find all batch-files
        % batches     = dir(fullfile(pathRawBatch, 'batch_*.m'));
        % find all m-files
        
        batches     = dir(fullfile(pathRawBatch, '*.m'));
        batches     = {batches.name};
        nBatches    = numel(batches);
        
        if ~exist(pathBatch, 'dir')
            mkdir(pathBatch);
        end
        
        
        % Update absolute paths in all batch files and write them to code/batches
        % folder in current location
        for b = 1:nBatches
            update_absolute_paths( ...
                fullfile(pathRawBatch, batches{b}), ...
                {pathSubjectReplace, pathSpmReplace, pathStudyReplace, ...
                dirSubjectReplace}, ...
                {pathSubject, pathSpm, pathStudy, dirSubject}, ...
                fullfile(pathBatch, batches{b}) ...
                )
        end
    end
end
